@@ -5,14 +5,16 @@ import zipfile
 from abc import abstractmethod, ABCMeta
 from contextlib import contextmanager
 
+from pydocx import types
 from pydocx.utils import (
-    PydocxPrePorcessor,
-    get_list_style,
-    parse_xml_from_string,
-    find_first,
+    MulitMemoizeMixin,
+    PydocxPreProcessor,
     find_all,
     find_ancestor_with_tag,
+    find_first,
+    get_list_style,
     has_descendant_with_tag,
+    parse_xml_from_string,
 )
 from pydocx.exceptions import MalformedDocxException
 
@@ -24,6 +26,9 @@ logger = logging.getLogger("NewParser")
 EMUS_PER_PIXEL = 9525
 USE_ALIGNMENTS = True
 
+# https://en.wikipedia.org/wiki/Twip
+TWIPS_PER_POINT = 20
+
 JUSTIFY_CENTER = 'center'
 JUSTIFY_LEFT = 'left'
 JUSTIFY_RIGHT = 'right'
@@ -31,7 +36,18 @@ JUSTIFY_RIGHT = 'right'
 INDENTATION_RIGHT = 'right'
 INDENTATION_LEFT = 'left'
 INDENTATION_FIRST_LINE = 'firstLine'
-DISABLED_STYLE_VALUES = ['false', '0', 'none']
+
+OPC_TAG_RELATIONSHIP = 'Relationship'
+OPC_TAG_RELATIONSHIP_ATTR_ID = 'Id'
+OPC_TAG_RELATIONSHIP_ATTR_TARGET_MODE = 'TargetMode'
+OPC_TAG_RELATIONSHIP_ATTR_TARGET = 'Target'
+OPC_TAG_RELATIONSHIP_ATTR_TYPE = 'Type'
+
+OPC_RELATIONSHIP_TARGET_MODE_EXTERNAL = 'External'
+
+OPC_RELATIONSHIP_TYPE_OFFICEDOCUMENT = 'http://schemas.openxmlformats.org/officeDocument/2006/relationships/officeDocument'  # noqa
+OPC_RELATIONSHIP_TYPE_NUMBERING = 'http://schemas.openxmlformats.org/officeDocument/2006/relationships/numbering'  # noqa
+OPC_RELATIONSHIP_TYPE_STYLES = 'http://schemas.openxmlformats.org/officeDocument/2006/relationships/styles'  # noqa
 
 # Add some helper functions to Element to make it slightly more readable
 
@@ -46,42 +62,251 @@ def ZipFile(path):  # This is not needed in python 3.2+
     f.close()
 
 
-class DocxParser:
-    __metaclass__ = ABCMeta
-    pre_processor_class = PydocxPrePorcessor
+class OPCRelationship(object):
+    '''
+    In Open Packaging Convention, a relationship is an association between a
+    part and another part, or a part and an external resource.
+    '''
 
-    def _extract_xml(self, f, xml_path):
+    def __init__(self, rId, rType, target_path, external, target=None):
+        self.rId = rId
+        self.rType = rType
+        self.target_path = target_path
+        self.target = target
+        self.external = external
+        if external:
+            self.container = None
+            self.filename = None
+            self.relationship_path = None
+        else:
+            self.container, self.filename = os.path.split(self.target_path)
+            self.relationship_path = os.path.join(
+                self.container,
+                '_rels',
+                '%s.rels' % self.filename,
+            )
+
+
+class OPCPart(object):
+    '''
+    In Open Packaging Convention, the term part corresponds to a file stored
+    within the ZIP archive.
+
+    A part may have relationships to other parts or external resources.
+    '''
+
+    def __init__(self, raw_data=None):
+        self.raw_data = raw_data
+        self._xml_tree = None
+        self.relationships_by_type = {}
+        self.relationships_by_id = {}
+
+    @property
+    def xml_tree(self):
+        if self._xml_tree is not None:
+            return self._xml_tree
+        if self.raw_data is not None:
+            self._xml_tree = parse_xml_from_string(self.raw_data)
+        return self._xml_tree
+
+    def get_relationship_by_type(self, name):
+        return self.relationships_by_type.get(name)
+
+    def get_relationship_by_id(self, rId):
+        return self.relationships_by_id.get(rId)
+
+    def add_relationship(self, relationship):
+        self.relationships_by_id[relationship.rId] = relationship
+        self.relationships_by_type[relationship.rType] = relationship
+
+
+class WordprocessingMLPackage(object):
+    '''
+    An interface to a WordprocessingML package
+
+    Example:
+
+    package = WordprocessingMLPackage.load('path/file.docx')
+    for el in package.get_relationship('officeDocument').target.xml_tree:
+        pass
+    '''
+    @staticmethod
+    def load(path_to_archive):
+        package = WordprocessingMLPackage()
+        with ZipFile(path_to_archive) as zip_file_handle:
+            return package._load(
+                file_handle=zip_file_handle,
+            )
+
+    def handle_required_resource_is_missing(self, resource):
+        raise MalformedDocxException(
+            'A required resource is missing: %s' % resource
+        )
+
+    def _read_resource_from_file(self, file_handle, resource_path):
         try:
-            return f.read(xml_path)
+            return file_handle.read(resource_path)
         except KeyError:
             return None
 
-    def _build_data(self, path, *args, **kwargs):
-        with ZipFile(path) as f:
-            # These must be in the ZIP in order for the docx to be valid.
-            self.document_text = f.read('word/document.xml')
-            self.relationship_text = f.read('word/_rels/document.xml.rels')
+    def _load(self, file_handle):
+        root = OPCRelationship(
+            rId=None,
+            rType='',
+            target_path='',
+            external=False,
+            target=OPCPart(),
+        )
 
-            # These are all optional.
-            self.styles_text = self._extract_xml(f, 'word/styles.xml')
-            self.fonts = self._extract_xml(f, 'word/fontTable.xml')
-            self.numbering_text = self._extract_xml(f, 'word/numbering.xml')
-            self.comment_text = self._extract_xml(f, 'word/comments.xml')
+        relationship_digraph = self._load_relationships(
+            file_handle=file_handle,
+            root=root,
+        )
 
-            zipped_image_files = [
-                e for e in f.infolist()
-                if e.filename.startswith('word/media/')
-            ]
-            for e in zipped_image_files:
-                self._image_data[e.filename] = f.read(e.filename)
+        self._build_and_associate_parts(
+            file_handle=file_handle,
+            relationship_digraph=relationship_digraph,
+            root=root,
+        )
+        return root.target
+
+    def _build_and_associate_parts(
+        self,
+        file_handle,
+        relationship_digraph,
+        root,
+    ):
+        target_path_to_part_map = {}
+        # Load data for the individual parts
+        for parent, relationships in relationship_digraph.items():
+            for relationship in relationships.values():
+                if relationship.external:
+                    # External relationships don't have content in the archive,
+                    # so we skip building a part
+                    continue
+
+                if relationship.target is None:
+                    # The part for this endpoint may have already been created.
+                    part = target_path_to_part_map.get(
+                        relationship.target_path,
+                    )
+                    relationship.target = part
+                    # If it wasn't created, this is just being set to None
+                    # again
+
+                if relationship.target is None:
+                    # Create the part for this endpoint
+                    raw_data = self._read_resource_from_file(
+                        file_handle=file_handle,
+                        resource_path=relationship.target_path,
+                    )
+                    part = OPCPart(raw_data=raw_data)
+                    # Maintain a list of created parts so we only create one
+                    # for each local endpoint
+                    target_path_to_part_map[relationship.target_path] = part
+                    relationship.target = part
+
+        # Add relationships to parts
+        for parent, relationships in relationship_digraph.items():
+            for relationship in relationships.values():
+                parent.target.add_relationship(relationship)
+
+    def _load_relationships(self, file_handle, root):
+        digraph_of_relationships = {}
+        stack = [root]
+        while len(stack) > 0:
+            relationship = stack.pop()
+            data = self._read_resource_from_file(
+                file_handle=file_handle,
+                resource_path=relationship.relationship_path,
+            )
+            if data is None:
+                continue
+            xml = parse_xml_from_string(data)
+            relationships = self._parse_relationship_xml(
+                xml=xml,
+                source=relationship,
+            )
+            digraph_of_relationships[relationship] = relationships
+            stack.extend(relationships.values())
+
+        if not digraph_of_relationships:
+            self.handle_required_resource_is_missing(
+                root.relationships_path,
+            )
+        return digraph_of_relationships
+
+    def _parse_relationship_xml(self, xml, source):
+        relationships = {}
+        for child in xml:
+            if child.tag == OPC_TAG_RELATIONSHIP:
+                relationship = self._create_relationship_from_element(
+                    element=child,
+                    source=source,
+                )
+                relationships[relationship.rId] = relationship
+        return relationships
+
+    def _create_relationship_from_element(self, element, source):
+        rid = element.get(OPC_TAG_RELATIONSHIP_ATTR_ID)
+        target_mode = element.get(OPC_TAG_RELATIONSHIP_ATTR_TARGET_MODE)
+        external = False
+        target_path = element.get(OPC_TAG_RELATIONSHIP_ATTR_TARGET)
+        if target_mode == OPC_RELATIONSHIP_TARGET_MODE_EXTERNAL:
+            external = True
+        else:
+            target_path = os.path.join(
+                source.container,
+                target_path,
+            )
+        return OPCRelationship(
+            rId=rid,
+            rType=element.get(OPC_TAG_RELATIONSHIP_ATTR_TYPE),
+            target_path=target_path,
+            external=external,
+        )
+
+
+class DocxParser(MulitMemoizeMixin):
+    __metaclass__ = ABCMeta
+    pre_processor_class = PydocxPreProcessor
+
+    def __init__(
+        self,
+        path,
+        convert_root_level_upper_roman=False,
+    ):
+        self.path = path
+        self._parsed = ''
+        self.block_text = ''
+        self.page_width = 0
+        self.convert_root_level_upper_roman = convert_root_level_upper_roman
+        self.pre_processor = None
+        self.visited = set()
+        self.list_depth = 0
+
+    def _load(self):
+        package = WordprocessingMLPackage.load(self.path)
+
+        self.document = package.get_relationship_by_type(
+            OPC_RELATIONSHIP_TYPE_OFFICEDOCUMENT,
+        ).target
 
-        self.root = parse_xml_from_string(self.document_text)
         self.numbering_root = None
-        if self.numbering_text:
-            self.numbering_root = parse_xml_from_string(self.numbering_text)
-        self.comment_root = None
-        if self.comment_text:
-            self.comment_root = parse_xml_from_string(self.comment_text)
+        numbering = self.document.get_relationship_by_type(
+            OPC_RELATIONSHIP_TYPE_NUMBERING,
+        )
+        if numbering:
+            self.numbering_root = numbering.target.xml_tree
+
+        pgSzEl = find_first(self.document.xml_tree, 'pgSz')
+        if pgSzEl is not None:
+            # pgSz is defined in twips, convert to points
+            pgSz = int(pgSzEl.attrib['w'])
+            self.page_width = pgSz / TWIPS_PER_POINT
+
+        self.styles_dict = self._parse_styles()
+        self.parse_begin(self.document.xml_tree)  # begin to parse
 
     def _parse_run_properties(self, rPr):
         """
@@ -111,11 +336,13 @@ class DocxParser:
         return run_properties
 
     def _parse_styles(self):
-        if self.styles_text is None:
+        styles = self.document.get_relationship_by_type(
+            OPC_RELATIONSHIP_TYPE_STYLES,
+        )
+        if not styles:
             return {}
-        tree = parse_xml_from_string(self.styles_text)
         styles_dict = {}
-        for style in find_all(tree, 'style'):
+        for style in find_all(styles.target.xml_tree, 'style'):
             style_val = find_first(style, 'name').attrib['val']
             run_properties = find_first(style, 'rPr')
             styles_dict[style.attrib['styleId']] = {
@@ -126,48 +353,13 @@ class DocxParser:
             }
         return styles_dict
 
-    def _parse_rels_root(self):
-        tree = parse_xml_from_string(self.relationship_text)
-        rels_dict = {}
-        for el in tree:
-            rId = el.get('Id')
-            target = el.get('Target')
-            rels_dict[rId] = target
-        return rels_dict
-
-    def __init__(
-            self,
-            path,
-            convert_root_level_upper_roman=False,
-            *args,
-            **kwargs):
-        self._parsed = ''
-        self.block_text = ''
-        self.page_width = 0
-        self.convert_root_level_upper_roman = convert_root_level_upper_roman
-        self._image_data = {}
-        self._build_data(path, *args, **kwargs)
-        self.pre_processor = None
-
-        #divide by 20 to get to pt (Office works in 20th's of a point)
-        """
-        see http://msdn.microsoft.com/en-us/library/documentformat
-        .openxml.wordprocessing.indentation.aspx
-        """
-        if find_first(self.root, 'pgSz') is not None:
-            self.page_width = int(
-                find_first(self.root, 'pgSz').attrib['w']
-            ) / 20
-
-        #all blank when we init
-        self.comment_store = None
-        self.visited = []
-        self.list_depth = 0
-        self.rels_dict = self._parse_rels_root()
-        self.styles_dict = self._parse_styles()
-        self.parse_begin(self.root)  # begin to parse
-
     def parse_begin(self, el):
+        self.populate_memoization({
+            'find_all': find_all,
+            'find_first': find_first,
+            'has_descendant_with_tag': has_descendant_with_tag,
+            '_get_tcs_in_column': self._get_tcs_in_column,
+        })
         self.pre_processor = self.pre_processor_class(
             convert_root_level_upper_roman=self.convert_root_level_upper_roman,
             styles_dict=self.styles_dict,
@@ -179,7 +371,7 @@ class DocxParser:
     def parse(self, el):
         if el in self.visited:
             return ''
-        self.visited.append(el)
+        self.visited.add(el)
         parsed = ''
         for child in el:
             # recursive. So you can get all the way to the bottom
@@ -216,7 +408,7 @@ class DocxParser:
             return parsed
 
     def parse_page_break(self, el, text):
-        #TODO figure out what parsed is getting overwritten
+        # TODO figure out what parsed is getting overwritten
         return self.page_break()
 
     def parse_table(self, el, text):
@@ -351,33 +543,26 @@ class DocxParser:
         if paragraph_tag_property is None:
             return text
 
-        _justification = paragraph_tag_property.find('jc')
+        jc = paragraph_tag_property.find('jc')
         indentation = paragraph_tag_property.find('ind')
-        if _justification is None and indentation is None:
+        if jc is None and indentation is None:
             return text
         alignment = None
         right = None
         left = None
         firstLine = None
-        if _justification is not None:  # text alignments
-            value = _justification.attrib['val']
+        if jc is not None:  # text alignments
+            value = jc.attrib['val']
             if value in [JUSTIFY_LEFT, JUSTIFY_CENTER, JUSTIFY_RIGHT]:
                 alignment = value
 
         if indentation is not None:
             if INDENTATION_RIGHT in indentation.attrib:
-                right = indentation.attrib[INDENTATION_RIGHT]
-                # divide by 20 to get to pt. multiply by (4/3) to get to px
-                right = (int(right) / 20) * float(4) / float(3)
-                right = str(right)
+                right = int(indentation.attrib[INDENTATION_RIGHT])
             if INDENTATION_LEFT in indentation.attrib:
-                left = indentation.attrib[INDENTATION_LEFT]
-                left = (int(left) / 20) * float(4) / float(3)
-                left = str(left)
+                left = int(indentation.attrib[INDENTATION_LEFT])
             if INDENTATION_FIRST_LINE in indentation.attrib:
-                firstLine = indentation.attrib[INDENTATION_FIRST_LINE]
-                firstLine = (int(firstLine) / 20) * float(4) / float(3)
-                firstLine = str(firstLine)
+                firstLine = int(indentation.attrib[INDENTATION_FIRST_LINE])
         if any([alignment, firstLine, left, right]):
             return self.indent(text, alignment, firstLine, left, right)
         return text
@@ -417,7 +602,7 @@ class DocxParser:
         if self.pre_processor.previous(next_el) is None:
             return False
         tag_is_inline_like = any(
-            has_descendant_with_tag(next_el, tag) for
+            self.memod_tree_op('has_descendant_with_tag', next_el, tag) for
             tag in inline_like_tags
         )
         if tag_is_inline_like:
@@ -478,7 +663,20 @@ class DocxParser:
         # Create the actual li element
         return self.list_element(parsed)
 
+    def _get_tcs_in_column(self, tbl, column_index):
+        return [
+            tc for tc in self.memod_tree_op('find_all', tbl, 'tc')
+            if self.pre_processor.column_index(tc) == column_index
+        ]
+
     def _get_rowspan(self, el, v_merge):
+        restart_in_v_merge = False
+        if v_merge is not None and 'val' in v_merge.attrib:
+            restart_in_v_merge = 'restart' in v_merge.attrib['val']
+
+        if not restart_in_v_merge:
+            return ''
+
         current_row = self.pre_processor.row_index(el)
         current_col = self.pre_processor.column_index(el)
         rowspan = 1
@@ -488,24 +686,20 @@ class DocxParser:
         # than the current_row and that are on the current_col
         if tbl is None:
             return ''
-        tcs = [
-            tc for tc in find_all(tbl, 'tc')
-            if self.pre_processor.row_index(tc) >= current_row and
-            self.pre_processor.column_index(tc) == current_col
-        ]
-        restart_in_v_merge = False
-        if v_merge is not None and 'val' in v_merge.attrib:
-            restart_in_v_merge = 'restart' in v_merge.attrib['val']
 
-        def increment_rowspan(tc):
-            if not restart_in_v_merge:
-                return False
+        tcs = [
+            tc for tc in self.memod_tree_op(
+                '_get_tcs_in_column', tbl, current_col,
+            ) if self.pre_processor.row_index(tc) >= current_row
+        ]
+
+        def should_increment_rowspan(tc):
             if not self.pre_processor.vmerge_continue(tc):
                 return False
             return True
 
         for tc in tcs:
-            if increment_rowspan(tc):
+            if should_increment_rowspan(tc):
                 rowspan += 1
             else:
                 rowspan = 1
@@ -517,7 +711,7 @@ class DocxParser:
         grid_span = find_first(el, 'gridSpan')
         if grid_span is None:
             return ''
-        return find_first(el, 'gridSpan').attrib['val']
+        return grid_span.attrib['val']
 
     def parse_table_cell_contents(self, el, text):
         parsed = text
@@ -530,10 +724,11 @@ class DocxParser:
 
     def parse_hyperlink(self, el, text):
         rId = el.get('id')
-        href = self.rels_dict.get(rId)
-        if not href:
+        relationship = self.document.get_relationship_by_id(rId)
+        if not relationship:
+            # Preserve the text even if we are unable to resolve the hyperlink
             return text
-        href = self.escape(href)
+        href = self.escape(relationship.target_path)
         return self.hyperlink(text, href)
 
     def _get_image_id(self, el):
@@ -586,25 +781,17 @@ class DocxParser:
     def parse_image(self, el):
         x, y = self._get_image_size(el)
         rId = self._get_image_id(el)
-        src = self.rels_dict.get(rId)
-        if not src:
+        relationship = self.document.get_relationship_by_id(rId)
+        if (not relationship or
+                not relationship.target or
+                not relationship.target.raw_data):
             return ''
-        src = os.path.join(
-            'word',
-            src,
+        return self.image(
+            relationship.target.raw_data,
+            relationship.filename,
+            x,
+            y,
         )
-        if src in self._image_data:
-            filename = os.path.split(src)[-1]
-            return self.image(self._image_data[src], filename, x, y)
-        return ''
-
-    def _is_style_on(self, value):
-        """
-        For b, i, u (bold, italics, and underline) merely having the tag is not
-        sufficient. You need to check to make sure it is not set to "false" as
-        well.
-        """
-        return value not in DISABLED_STYLE_VALUES
 
     def parse_t(self, el, parsed):
         if el.text is None:
@@ -612,7 +799,7 @@ class DocxParser:
         return self.escape(el.text)
 
     def parse_tab(self, el, parsed):
-        return ' '
+        return self.tab()
 
     def parse_hyphen(self, el, parsed):
         return '-'
@@ -640,7 +827,7 @@ class DocxParser:
 
         # Get the rPr for the current style, they are the defaults.
         p = find_ancestor_with_tag(self.pre_processor, el, 'p')
-        paragraph_style = find_first(p, 'pStyle')
+        paragraph_style = self.memod_tree_op('find_first', p, 'pStyle')
         if paragraph_style is not None:
             style = paragraph_style.get('val')
             style_defaults = self.styles_dict.get(style, {})
@@ -656,6 +843,18 @@ class DocxParser:
             )
             run_properties.update(local_run_properties)
 
+        inline_tag_types = {
+            'b': types.OnOff,
+            'i': types.OnOff,
+            'u': types.Underline,
+            'caps': types.OnOff,
+            'smallCaps': types.OnOff,
+            'strike': types.OnOff,
+            'dstrike': types.OnOff,
+            'vanish': types.OnOff,
+            'webHidden': types.OnOff,
+        }
+
         inline_tag_handlers = {
             'b': self.bold,
             'i': self.italics,
@@ -679,8 +878,8 @@ class DocxParser:
                     styles_needing_application.append(self.subscript)
             else:
                 if (
-                        property_name in inline_tag_handlers and
-                        self._is_style_on(property_value)
+                    property_name in inline_tag_handlers and
+                    inline_tag_types.get(property_name)(property_value)
                 ):
                     styles_needing_application.append(
                         inline_tag_handlers[property_name],
@@ -694,6 +893,8 @@ class DocxParser:
 
     @property
     def parsed(self):
+        if not self._parsed:
+            self._load()
         return self._parsed
 
     @property
@@ -721,12 +922,12 @@ class DocxParser:
         return text
 
     @abstractmethod
-    def image_handler(self, path):
+    def image_handler(self, image_data, path):
         return path
 
     @abstractmethod
     def image(self, data, filename, x, y):
-        return self.image_handler(data)
+        return self.image_handler(data, filename)
 
     @abstractmethod
     def deletion(self, text, author, date):
@@ -801,5 +1002,12 @@ class DocxParser:
         return True
 
     @abstractmethod
-    def indent(self, text, left='', right='', firstLine=''):
-        return text  # TODO JUSTIFIED JUSTIFIED TEXT
+    def indent(
+        self,
+        text,
+        alignment=None,
+        left=None,
+        right=None,
+        firstLine=None,
+    ):
+        return text
